package net.sabi.pester;

import danger.app.Application;
import danger.app.Event;
import danger.internal.Date;
import danger.ui.ActiveListView;
import danger.ui.Button;
import danger.ui.CheckBox;
import danger.ui.DateTimeEditor;
import danger.ui.DatePicker;
import danger.ui.DialogWindow;
import danger.ui.NumberEditor;
import danger.ui.PopupMenu;
import danger.ui.RadioButton;
import danger.ui.Rect;
import danger.ui.TypeAheadTextField;
import danger.util.ActiveList;
import danger.util.StdActiveList;
import danger.util.DEBUG;

public class AlarmListView extends ActiveListView
    implements Resources, Commands {
    private static Pester sPester;
    private static AlarmListWindow sAlarmListWindow;
    // XXX some of these shouldn't conceptually be static, but who cares
    private static StdActiveList sAlarmList;
    private static Alarms sAlarms;

    private static DialogWindow sEditWindow;
    private TypeAheadTextField sMessageField;
    private RadioButton sInButton, sAtButton;
    private NumberEditor sPeriodField;
    private PopupMenu sPeriodUnitsPopup;
    private CheckBox sRepeatCheckBox;
    private DateTimeEditor sTimeEditor, sDateEditor;
    private DatePicker sDatePicker;
    private Button sDiscardAlarmButton, sSetAlarmButton;

    private static Alarm sFocusedAlarm, sEditingAlarm;

    private static final int EVENT_CANCEL_ALARM = -1; // XXX move this?
    
    public void onDecoded() {
	sPester = (Pester)Application.getCurrentApp();
	sAlarmListWindow = (AlarmListWindow)getWindow();
	sAlarmList = new StdActiveList();
	setList(sAlarmList);

	sEditWindow = sPester.getDialog(ID_ALARM_SET_DIALOG, this);
	sEditWindow.setCancelButtonEvent(new Event(this, EVENT_CANCEL_ALARM));
	sMessageField = (TypeAheadTextField)sEditWindow.getDescendantWithID(ID_MESSAGE_FIELD);
	sEditWindow.disableBottomRightButtonOnEmptyField(sMessageField);
	sInButton = (RadioButton)sEditWindow.getDescendantWithID(ID_IN_BUTTON);
	sAtButton = (RadioButton)sEditWindow.getDescendantWithID(ID_AT_BUTTON);
	sPeriodField = (NumberEditor)sEditWindow.getDescendantWithID(ID_PERIOD_FIELD);
	sPeriodUnitsPopup = (PopupMenu)sEditWindow.getDescendantWithID(ID_PERIOD_UNITS_POPUP);
	sRepeatCheckBox = (CheckBox)sEditWindow.getDescendantWithID(ID_PERIOD_REPEAT_CHECKBOX);
	sTimeEditor = (DateTimeEditor)sEditWindow.getDescendantWithID(ID_TIME_EDITOR);
	sDateEditor = (DateTimeEditor)sEditWindow.getDescendantWithID(ID_DATE_EDITOR);
	sDatePicker = (DatePicker)sEditWindow.getDescendantWithID(ID_DATE_PICKER);
	sSetAlarmButton = (Button)sEditWindow.getDescendantWithID(ID_SET_ALARM_BUTTON);
	sDiscardAlarmButton = (Button)sEditWindow.getDescendantWithID(ID_DISCARD_ALARM_BUTTON);
	sInButton.setValue(1);
	sDateEditor.limitToDangerEpoch();

	super.onDecoded();
    }

    public boolean eventWidgetUp(int widget, Event e) {
	switch (widget) {
	case Event.DEVICE_BUTTON_BACK:
	case Event.DEVICE_BUTTON_CANCEL:
	    sPester.returnToLauncher();
	    return true;
	}
	return super.eventWidgetUp(widget, e);
    }

    protected void editDateOrPeriod(Alarm alarm) {
        boolean usesPeriod = alarm.getUsesPeriod();
	if (usesPeriod) {
	    int period = alarm.getPeriod();
	    int unitsPerSec = 1;
	    DEBUG.p("period is " + period);
	    for (int i = sPeriodUnitsPopup.itemCount() - 1 ; i >= 0 ; --i) {
		unitsPerSec =
		    sPeriodUnitsPopup.getIDOfItem(sPeriodUnitsPopup.getItem(i));
		DEBUG.p("unitsPerSec = " + unitsPerSec);
		if (period % unitsPerSec == 0) break;
	    }
	    sPeriodField.setValue(period / unitsPerSec);
	    sPeriodUnitsPopup.setValueWithItemID(unitsPerSec);
	    sRepeatCheckBox.setValue(alarm.getRepeating() ? 1 : 0);
	}

	Date date = alarm.getDate();
	sTimeEditor.setDate(date);
	sDateEditor.setDate(date);
	sDatePicker.setDate(date);

	sInButton.setValue(usesPeriod ? 1 : 0);
	sAtButton.setValue(usesPeriod ? 0 : 1);
    }

    protected void editAlarm(Alarm alarm, boolean asNew) {
	sEditingAlarm = alarm;
	alarm.beginEditing();
	sMessageField.setText(alarm.getMessage());
	if (asNew) {
	    sDiscardAlarmButton.hide();
	} else {
	    sDiscardAlarmButton.show();
	    editDateOrPeriod(alarm);
	}
	constrainDate();
	sEditWindow.show();
    }

    protected Date editingDate() {
	Date date = sDateEditor.getDate();
	date.setTime(sTimeEditor.getDate().getTime());
	return date;
    }

    protected void constrainDate() {
	// XXX schedule every minute (second?) if in AT mode
	Date now = new Date();
	sDateEditor.min(now);
	sDatePicker.min(now);
	sSetAlarmButton.setEnabled(editingDate().compareTo(now) > 0);
    }

    public boolean receiveEvent(Event e) {
	switch (e.type) {
	case EVENT_NEW_ALARM:
	    editAlarm(new Alarm(), true);
	    return true;
	case EVENT_SET_ALARM:
	    sEditingAlarm.setMessage(sMessageField.toString());
	    if (sInButton.getValue() == 1) {
		DEBUG.p("period: " + sPeriodField.getValue());
		DEBUG.p(" units: " + sPeriodUnitsPopup.getValueAsItemID());
		sEditingAlarm.setPeriod(sPeriodField.getValue() * 
					sPeriodUnitsPopup.getValueAsItemID(),
					sRepeatCheckBox.getValue() == 1);
	    } else {
		sEditingAlarm.setDate(editingDate());
	    }
	    if (sEditingAlarm != sFocusedAlarm) { // new alarm
		sAlarmList.addItem(sEditingAlarm);
		setFocusedItem(sEditingAlarm);
	    } else {
		sEditingAlarm.update();
	    }
	case EVENT_CANCEL_ALARM:
	    // XXX (re)schedule alarm
	    sEditingAlarm.endEditing();
	    return true;
	case EVENT_DISCARD_ALARM:
	    // XXX confirmation
	    sAlarmList.removeItem(sFocusedAlarm);
	    return true;
	case EVENT_IN:
	    sAtButton.setValue(0);
	    return true;
	case EVENT_AT:
	    sInButton.setValue(0);
	    constrainDate();
	    return true;
	case EVENT_TIME_EDITOR:
	    Date date = editingDate();
	    // XXX factor this out (it's used twice)
	    // XXX do this on exit from the field, not on change
	    if (date.compareTo(new Date()) <= 0) {
		date.addDays(1);
		sDatePicker.setDate(date);
		sDateEditor.setDate(date);
	    }
	    constrainDate();
	    return true;
        case EVENT_DATE_EDITOR:
	    sDatePicker.setDate(sDateEditor.getDate());
	    constrainDate();
	    return true;
        case EVENT_DATE_PICKER:
	    sDateEditor.setDate((Date)e.argument);
	    constrainDate();
	    return true;
	}
	return super.receiveEvent(e);
    }

    protected void itemActivated(Object item) {
	// XXX deschedule alarm
	editAlarm(sFocusedAlarm, false);
    }
    protected void itemFocused(Object item) {
	sFocusedAlarm = (Alarm)item;
    }

    public int alarmsSet() {
	return getListSize();
    }
    public void onItemAdded(ActiveList list, Object item, int index) {
	sAlarmListWindow.updateAlarmCount();
	super.onItemAdded(list, item, index);
    }
    public void onItemRemoved(ActiveList list, Object item, int index) {
	sAlarmListWindow.updateAlarmCount();
	super.onItemRemoved(list, item, index);
    }

    protected CharSequence getToolTipForItem(Object item) {
	if (item == null) // this is dumb, why do we get asked for a tooltip?
	    return null;
	// XXX only set a tooltip if the message is truncated
	return ((Alarm)item).getMessage();
    }
}
